package dictionary;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import states.Tile;

public class Trie {
	public TrieNode root;

	/**
	 * Constructor
	 */
	public Trie() {
		root = new TrieNode();
	}

	/**
	 * Adds a word to the Trie
	 * 
	 * @param word
	 */
	public void addWord(String word) {
		root.addWord(word.toLowerCase());

	}

	/**
	 * Get the words in the Trie with the given prefix
	 * 
	 * @return a List containing String objects containing the words in the Trie
	 *         with the given prefix.
	 */
	public List<String> getWords(String prefix) {
		TrieNode lastNode = root;
		if (prefix != null && prefix != " ") {
			prefix = prefix.toLowerCase();

			// Find the node which represents the last letter of the prefix
			for (int i = 0; i < prefix.length(); i++) {
				lastNode = lastNode.getNode(prefix.charAt(i));
				// System.out.println("lastNode" + lastNode);
				// If no node matches, then no words exist, return empty list
				if (lastNode == null) {
					System.out.println(" No word matches");
					return new ArrayList();
				}
			}

			// Return the words which eminate from the last node
			System.out.println("\nWords in trie");
			List<String> list1 = lastNode.getWords();
			System.out.println(list1);

		}

		else {
			if (prefix == null)
				System.out.println("Cannot retrieve null value ");
			else

				System.out.println("Cannot retrieve empty string ");
		}

		return lastNode.getWords();
	}

	public Set<String> getPossibleWords(String boardRack) {
		List<String> sortedrackLettersl = new ArrayList<String>();
		List<TrieNode> childrenWords = new ArrayList<TrieNode>();
		List<TrieNode> validWords = new ArrayList<TrieNode>();
		Set<TrieNode> validWordSet = new HashSet<TrieNode>();
		System.out.println("\nCombine board and rack letters :\t" + boardRack);
		boardRack = boardRack.trim();
		boardRack = boardRack.toLowerCase();
		TrieNode tempNode = null;
		TrieNode tempNode1 = null;
		String boardLetter = boardRack.charAt(0) + "";
		String rackLetters = boardRack.substring(1, boardRack.length());
		char[] split = rackLetters.toCharArray();
		Arrays.sort(split);
		String sortedrackLetters = new String(split);

		for (int i = 0; i < sortedrackLetters.length(); i++) {
			sortedrackLettersl.add(sortedrackLetters.charAt(i) + "");
		}
		// System.out.println("sortedrackLetters :" + sortedrackLetters);
		String sortedboardRack = boardLetter + sortedrackLetters;
		// System.out.println("sortedboardRack :" + sortedboardRack);
		String temp = "";
		Set<String> permutationWords = permutation(sortedboardRack, boardLetter);
		System.out.println("\nPermutations of the board & rack letters :");
		// System.out.println(permutationWords);
		for (String s : permutationWords) {
			TrieNode lastNode = root;
			// System.out.println("------------ Permuted WORD -------->>>" + s);

			if (rackLetters != null && rackLetters != " ") {
				// Find the node which represents the last letter of the prefix
				for (int i = 0; i < s.length() - 1; i++) {
					// System.out.println(" i " + i + " Str:" + s);
					tempNode = lastNode;
					if (lastNode != root && tempNode == null) {
						// System.out.println(" No word matches");
						break;
					}
					lastNode = lastNode.getNode(s.charAt(i));
					tempNode = lastNode;
					// System.out.println("----Tempnode " + tempNode);
					if (lastNode == null || tempNode == null) {
						// System.out.println(" No word matches");
						break;
					} else {
						validWords = tempNode.getWords1(sortedrackLettersl, validWords);
						// System.out.println("/nReturned validWords: " +
						// validWords);
						List<TrieNode> validword = tempNode.getWords1(sortedrackLettersl, validWords);
						for (TrieNode w : validword) {
							if (StringUtils.contains(w + "", boardLetter))
								validWordSet.addAll(validword);
						}

						// System.out.println("*******Possible words List: " +
						// validWordSet);
					}

				}

			}

			else {
				if (boardRack == null)
					System.out.println("Cannot retrieve null value ");
				else

					System.out.println("Cannot retrieve empty string ");
			}
		}
		Set<String> finalWordList = new HashSet<String>();
		System.out.println("\nFINAL Valid words");

		for (TrieNode w : validWordSet) {
			finalWordList.add(w + "");
		}
		System.out.println(finalWordList);
		return finalWordList;

	}

	public static Set<String> permutation(String str, String boardLetter) {
		Set<String> permutationWords = new HashSet<String>();
		permutation("", str, permutationWords, boardLetter);
		// for (String s : permutationWords)
		// System.out.println(s);
		return permutationWords;
	}

	private static void permutation(String prefix, String str, Set<String> permutationWords, String boardLetter) {
		int n = str.length();
		if (n == 0) {
			// System.out.println(prefix);
			if (StringUtils.contains(prefix, boardLetter)) {
				// System.out.println(prefix + " *****" + boardLetter);
				permutationWords.add(prefix);
			}

		} else {
			for (int i = 0; i < n; i++)
				permutation(prefix + str.charAt(i), str.substring(0, i) + str.substring(i + 1, n), permutationWords,
						boardLetter);
		}

	}
}