package dictionary;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class TrieNode {
	private TrieNode parent;
	private TrieNode[] children;
	private boolean isLeaf; // Quick way to check if any children exist
	private boolean isWord; // Does this node represent the last character of a
							// word
	private char character; // The character this node represents

	/**
	 * Constructor for top level root node.
	 */
	public TrieNode() {
		children = new TrieNode[26]; // for each alphabet
		isLeaf = true;
		isWord = false;
	}

	/**
	 * Constructor for child node.
	 */
	public TrieNode(char character) {
		this();
		this.character = character;
	}

	/**
	 * Adds a word to this node. This method is called recursively and adds
	 * child nodes for each successive letter in the word, therefore recursive
	 * calls will be made with partial words.
	 * 
	 * @param word
	 *            the word to add
	 */
	public void addWord(String word) {
		isLeaf = false;
		// System.out.println(word.charAt(0));
		word = word.toLowerCase();
		int charPos = word.charAt(0) - 'a';
		// System.out.println("CharPos" + charPos);

		if (children[charPos] == null) {
			children[charPos] = new TrieNode(word.charAt(0));
			children[charPos].parent = this;
		}

		if (word.length() > 1) {
			children[charPos].addWord(word.substring(1));
		} else {
			children[charPos].isWord = true;
		}

	}

	/**
	 * Returns the child TrieNode representing the given char, or null if no
	 * node exists.
	 * 
	 * @param c
	 * @return
	 */
	protected TrieNode getNode(char c) {

		// System.out.println(" char " + c);

		int ascii = (int) c - 97;

		if (children[ascii] == null) {
			// System.out.println("Null");
			return null;
		}
		return children[c - 'a'];
	}

	/**
	 * Returns a List of String objects which are lower in the hierarchy than
	 * this node.
	 * 
	 * @return
	 */
	protected List<String> getWords() {
		// Create a list to return
		List<String> list = new ArrayList<String>();
		List<String> word = new ArrayList<String>();

		// If this node represents a word, add it
		if (isWord) {
			list.add(toString());
		}

		// If any children
		if (!isLeaf) {
			// Add any words belonging to any children
			for (int i = 0; i < children.length; i++) {
				// System.out.println(" children[i] " + children[i]);
				if (children[i] != null) {
					list.addAll(children[i].getWords());
					break;

				}
				// System.out.println(children.toString());
			}

		}

		return list;

	}

	public List<TrieNode> getWords1(List<String> sortedrackLettersl, List<TrieNode> validWords) {

		List<String> list = new ArrayList<String>();
		List<TrieNode> validWordNodes = new ArrayList<TrieNode>();
		List<String> tempsortedrackLettersl = sortedrackLettersl;
		int k = 0;
		if (!isLeaf) {
			boolean f = false;
			// Add any words belonging to any children
			for (String s : tempsortedrackLettersl) {
				for (int i = 0; i < children.length; i++) {
					k++;
					if (children[i] != null) {
						String child = children[i] + "";
						TrieNode childNode = children[i];
						// System.out.println(" Child in tree : " + child + "
						// ,\tSortedrack: " + s);
						if (child != null || child != "null") {
							if (s.contains(child.substring(child.length() - 1))) {
								if (!list.contains(child)) {
									String childsub = child.substring(child.length() - 1);
									// System.out.println("\nChild " + child);
									// System.out.println(" Child substr " +
									// child.substring(child.length() - 1));
									list.add(child);
									if (childNode.isWord) {
										validWordNodes.add(childNode);
									}
									// tempsortedrackLettersl.remove(s);
									f = true;

									break;

								}

							}
						}

						// if (f == true) {
						// break;
						// }

					}

				}
			}
			// System.out.println("\nList " + list);
			if (k < 26) {
				for (String s : list) {
					String lastLetter = s.substring(s.length() - 1);

					// System.out.println("lastLetter " + lastLetter);
					/* Remove first word in list */
					// sortedrackLettersl.remove(lastLetter);
					// getWords1(s, sortedrackLettersl);
				}

			}

			// System.out.println("\nValidWords " + validWordNodes);

		}

		return validWordNodes;

	}

	/**
	 * 
	 * Gets the String that this node represents.
	 * 
	 * For example, if this node represents the character t, whose parent
	 * 
	 * represents the charater a, whose parent represents the character
	 * 
	 * c, then the String would be "cat".
	 * 
	 * @return
	 * 
	 */

	public String toString()

	{

		if (parent == null)

		{

			return "";

		}

		else

		{

			return parent.toString() + new String(new char[] { character });

		}

	}

}